# coding: utf-8
# From code by James Livingston



import totem
import gobject, gtk, gtk.glade
import sys
from settings import *
import time
import os
import subtitles
import pango
import prefs

class PowerTotem(totem.Plugin):

	def __init__(self):
		totem.Plugin.__init__(self)

	def make_subtitle_label(self, settings_object):

		self.Restored = False

		self.PreviousLast = False

		sub_parent = gtk.EventBox()
		sub_parent.show()
		sub_bg = gtk.gdk.Color(0, 0, 0)
		sub_parent.modify_bg(gtk.STATE_NORMAL, sub_bg)
		self.subtitle_label = gtk.Label('')

		if settings_object.get_setting('subtitles') == 'true':
			self.subtitle_label.show()

		self.subtitle_label.set_justify(gtk.JUSTIFY_CENTER)
		self.subtitle_label.set_use_markup(True)
		self.subtitle_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
		sub_parent.add(self.subtitle_label)

		self.video.pack_start(sub_parent, False, False, 0)
		self.video_widget.get_parent().modify_bg(gtk.STATE_NORMAL, sub_bg)
		self.video.reorder_child(sub_parent, 1)

	def set_show_border(self):
		self.video.get_children()[0].set_border_width(0)

	def set_show_sub_label(self, value):
		if value:
			self.subtitle_label.show()
		else:
			self.subtitle_label.hide()

	def set_show_menu(self, value):
		if value:
			self.menu.show()
			self.button.hide()
		else:
			self.menu.hide()
			self.button.show()

	def bring_back_menu(self, button, settings):
		self.set_show_menu(True)
		settings.save_setting('menu_visible', 'true')

	def activate(self, totem):

		self.EverySec = False
		self.shut_active = False
		self.NotRestore = False
		self.totem_sub = totem
		self.OwnSubs = False
		self.OwnSubName = ''

		video_widget = totem.get_video_widget()
		video_parent = video_widget.get_parent().get_parent()

		self.video = video_parent
		self.video_widget = video_widget
	
		settings_object = settings()


		self.make_subtitle_label(settings_object)





		self.set_sub_text(None)
		menu_widget = video_widget.get_parent().get_parent().get_parent()
		menu_widget = menu_widget.get_parent()

		main_vbox = menu_widget.get_children()[0]
		self.main_window = main_vbox.get_parent().get_parent()

		status_bar = self.main_window.get_children()[0].get_children()[-1]

		status_label = status_bar.get_children()[0].get_children()[0].get_children()[3]


		menu_widget = main_vbox.get_children()[0]
		self.menu = menu_widget

		controls = self.main_window.get_children()[0].get_children()[1].get_children()[0].get_children()[2].get_children()[1].get_children()


		try:
			type(controls)
		except:
			print "Error"
			return


		self.seek_bar = controls[0].get_parent().get_parent().get_children()[0].get_children()[1]

		self.time = self.seek_bar.get_parent().get_children()[0]

		def time_prop(widget, not_important):
			self.Restored = False
			if int(self.seek_bar.get_value()) == 65535:
				self.NotRestore = True

		self.hid4 = self.time.connect('state-changed', time_prop)
		

		self.hid5 = self.seek_bar.connect("value_changed", subtitles.sub_update, self.set_sub_text, status_label, self.main_window, settings_object, totem, self, controls[2].get_property("sensitive") == False)

		self.button = gtk.Button('')
		self.button.show()
		self.button.set_relief(gtk.RELIEF_NONE)
		image = gtk.Image()
		image.set_from_stock('gtk-undo', gtk.ICON_SIZE_MENU)
		self.button.set_image(image)
		self.button.connect("clicked", self.bring_back_menu, settings_object)

		self.seek_bar.get_parent().pack_start(self.button, False, False, 0)


		self.shut_button = gtk.ToggleButton('')
		self.shut_button.show()
		self.shut_button.set_relief(gtk.RELIEF_NONE)
		image2 = gtk.Image()
		image2.set_from_stock('gtk-stop', gtk.ICON_SIZE_MENU)
		self.shut_button.set_image(image2)
		self.shut_button.set_focus_on_click(False)
		self.shut_button.connect("clicked", self.shut_menu)

		self.seek_bar.get_parent().pack_start(self.shut_button, False, False, 0)

		if settings_object.get_setting('menu_visible') == 'true':
			self.set_show_menu(True)
		else:
			self.set_show_menu(False)




		edit_menu = menu_widget.get_children()[1].get_submenu()
		file_menu = menu_widget.get_children()[0].get_submenu()
		
		self.edit = edit_menu
		self.file = file_menu

		self.make_power_menu(edit_menu, file_menu, settings_object)
		self.make_power_toolbar(menu_widget)

		self.create_status_icon(totem, controls, menu_widget, settings_object)




		
		print "PowerTotem plugin activated."

	def set_show_status_icon(self, value):
		self.status_icon.set_visible(value)
		
	
	def deactivate(self, totem):
			try:
				self.status_icon.set_visible(False)
			except:
				pass

			self.power_separator.destroy()
			self.power_open_subs.destroy()
			self.power_preferences.destroy()
			self.subtitle_label.get_parent().destroy()
			self.subtitle_label.destroy()
			self.button.destroy()
			self.shut_button.destroy()
			self.Toolbar.destroy()

			self.status_icon.disconnect(self.hid1)
			self.status_icon.disconnect(self.hid2)
			self.main_window.disconnect(self.hid3)
			self.time.disconnect(self.hid4)
			self.seek_bar.disconnect(self.hid5)

			print "PowerTotem plugin deactivated."



	def create_status_icon(self,totem, controls, menu, settings_object):
		self.status_icon = gtk.status_icon_new_from_icon_name("totem")


		if settings_object.get_setting("tray_icon") == 'true':
			self.status_icon.set_visible(True)
		else:
			self.status_icon.set_visible(False)



		self.status_icon.set_tooltip(_("Totem Movie Player"))

		self.hid1 = self.status_icon.connect("activate", self.status_clicked, totem)
		self.hid2 = self.status_icon.connect("popup-menu", self.status_popup, totem, controls, self.main_window, menu)

	def make_power_toolbar(self, main_menu):
		self.Toolbar = gtk.Toolbar()
		ToolOpen = gtk.ToolButton('gtk-open')
		self.Toolbar.insert(ToolOpen, 0)

		ToolOpenSubs = gtk.ToolButton('gtk-open')
		ToolOpenSubs.set_label(_("Open subtitles..."))
		self.Toolbar.insert(ToolOpenSubs, 1)

		ToolPrefs = gtk.ToolButton('gtk-preferences')
		self.Toolbar.insert(ToolPrefs, 2)

		ToolClear = gtk.ToolButton('gtk-clear')
		ToolClear.set_label(_("_Clear Playlist"))
		self.Toolbar.insert(ToolClear, 3)
		ToolClear.set_use_underline(True)

		ToolFull = gtk.ToolButton('gtk-fullscreen')
		self.Toolbar.insert(ToolFull, 4)


		self.main_window.get_children()[0].get_children()[0].add(self.Toolbar)
		self.Toolbar.show_all()

		def toolbar_open(widget):
			func = main_menu.get_children()[0].get_submenu().get_children()[1]
			func.activate()

		def toolbar_open_subs(widget):
			func = main_menu.get_children()[0].get_submenu().get_children()[2]
			func.activate()

		def toolbar_fullscreen(widget):
			self.totem_sub.action_fullscreen_toggle()

		def toolbar_clear(widget):
			func = main_menu.get_children()[1].get_submenu().get_children()[6]
			func.activate()

		def toolbar_prefs(widget):
			func = main_menu.get_children()[1].get_submenu().get_children()[9]
			func.activate()

		ToolOpen.connect("clicked", toolbar_open)
		ToolOpenSubs.connect("clicked", toolbar_open_subs)
		ToolPrefs.connect("clicked", toolbar_prefs)
		ToolClear.connect("clicked", toolbar_clear)
		ToolFull.connect("clicked", toolbar_fullscreen)

		def disable_buttons(widget, nothing):
			ToolOpenSubs.set_sensitive(self.totem_sub.is_playing())
			ToolClear.set_sensitive(main_menu.get_children()[1].get_submenu().get_children(
)[6].get_property('sensitive'))
			self.set_show_border()

		self.hid3 = self.main_window.connect("expose-event", disable_buttons)

	def set_show_toolbar(self, value):
		if value:
			self.Toolbar.show()
		else:
			self.Toolbar.hide()


	def status_clicked(self, icon, totem):
		if not self.main_window.get_title() == 'minimized_totem':
			self.old_title = self.main_window.get_title()
			self.fullscreen = totem.is_fullscreen()
			if self.fullscreen:
				totem.action_fullscreen(False)

			self.old_window = [self.main_window.get_position(), self.main_window.get_size()]

			self.main_window.set_title('minimized_totem')

			self.main_window.hide()
		else:
			self.main_window.set_title(self.old_title)
			self.main_window.present()
			self.main_window.resize(self.old_window[1][0], self.old_window[1][1])
			self.main_window.move(self.old_window[0][0], self.old_window[0][1])

		

	def msg_box(self, text, parent):
		dialog = gtk.MessageDialog(parent, gtk.DIALOG_MODAL | \
		gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE,\
		text)


		dialog.set_title(_("PowerTotem"))
		dialog.run()
		dialog.destroy()

	def make_power_menu(self, edit_menu, file_menu, settings):
		self.power_separator = gtk.SeparatorMenuItem()
		self.power_separator.show()


		self.power_open_subs = gtk.ImageMenuItem(_("Open subtitles..."), True)
		power_open_subs_image = gtk.Image()
		power_open_subs_image.set_from_stock("gtk-open", gtk.ICON_SIZE_MENU)
		self.power_open_subs.set_image(power_open_subs_image)
		self.power_open_subs.show()

		self.power_open_subs.connect('activate', self.open_subtitles)

		self.power_preferences = gtk.ImageMenuItem(_("PowerTotem Preferences"), True)
		power_preferences_image = gtk.Image()
		power_preferences_image.set_from_stock("gtk-preferences", gtk.ICON_SIZE_MENU)
		self.power_preferences.set_image(power_preferences_image)
		self.power_preferences.show()

		self.power_preferences.connect("activate", self.preferences_window, self.main_window, settings)

		edit_menu.add(self.power_separator)
		edit_menu.add(self.power_preferences)
		file_menu.insert(self.power_open_subs, 2)

	def status_popup(self, status, v1, v2, totem, controls, window, main_menu):
		menu = gtk.Menu()

		show_window = gtk.CheckMenuItem(_("Show main window on screen"), True)

		if window.get_title() == "minimized_totem":
			show_window.set_active(False)
		else:
			show_window.set_active(True)

		sep1 = gtk.SeparatorMenuItem()
		sep2 = gtk.SeparatorMenuItem()
		sep3 = gtk.SeparatorMenuItem()
		sep4 = gtk.SeparatorMenuItem()
		sep5 = gtk.SeparatorMenuItem()

		if not totem.is_playing():
			play = gtk.ImageMenuItem(_("Play"), False)
			play_image = gtk.Image()
			play_image.set_from_stock("gtk-media-play", gtk.ICON_SIZE_MENU)
			play.set_image(play_image)
		else:
			play = gtk.ImageMenuItem(_("Pause"), False)
			play_image = gtk.Image()
			play_image.set_from_stock("gtk-media-pause", gtk.ICON_SIZE_MENU)
			play.set_image(play_image)

		open_file = gtk.ImageMenuItem("gtk-open", False)


		next = gtk.ImageMenuItem(_("Next"), False)
		next_image = gtk.Image()
		next_image.set_from_stock("gtk-media-next", gtk.ICON_SIZE_MENU)
		next.set_image(next_image)

		prev = gtk.ImageMenuItem(_("Previous"), False)
		prev_image = gtk.Image()
		prev_image.set_from_stock("gtk-media-previous", gtk.ICON_SIZE_MENU)
		prev.set_image(prev_image)

		fullscreen = gtk.ImageMenuItem(_("_Fullscreen"), False)
		fullscreen_image = gtk.Image()
		fullscreen_image.set_from_stock("gtk-fullscreen", gtk.ICON_SIZE_MENU)
		fullscreen.set_image(fullscreen_image)

		clear_playlist = gtk.ImageMenuItem(_("_Clear Playlist"), False)
		clear_playlist_image = gtk.Image()
		clear_playlist_image.set_from_stock("gtk-clear", gtk.ICON_SIZE_MENU)
		clear_playlist.set_image(clear_playlist_image)

		clear_playlist.set_sensitive(main_menu.get_children()[1].get_submenu().get_children()[6].get_property("sensitive"))


		plugins = gtk.MenuItem(_("Plugins..."), True)



		settings = gtk.ImageMenuItem(_("Prefere_nces"), False)
		settings_image = gtk.Image()
		settings_image.set_from_stock("gtk-preferences", gtk.ICON_SIZE_MENU)
		settings.set_image(settings_image)

		quit = gtk.ImageMenuItem("gtk-quit", False)

		open_func = main_menu.get_children()[0].get_submenu().get_children()[0]

		def open_active(self):
			func = main_menu.get_children()[0].get_submenu().get_children()[1]
			func.activate()

		def play_active(self):
			func = main_menu.get_children()[0].get_submenu().get_children()[14]
			func.activate()

		def next_active(self):
			totem.action_next()

		def prev_active(self):
			totem.action_previous()

		def fullscreen_active(self):
			totem.action_fullscreen_toggle()

		def clear_active(self):
			func = main_menu.get_children()[1].get_submenu().get_children()[6]
			func.activate()

		def plugins_active(self):
			func = main_menu.get_children()[1].get_submenu().get_children()[8]
			func.activate()

		def settings_active(self):
			func = main_menu.get_children()[1].get_submenu().get_children()[9]
			func.activate()

		def quit_active(self):
			totem.action_exit()


		show_window.connect("activate", self.status_clicked, totem)
		open_file.connect("activate", open_active)
		play.connect("activate", play_active)
		next.connect("activate", next_active)
		prev.connect("activate", prev_active)
		fullscreen.connect("activate", fullscreen_active)
		clear_playlist.connect("activate", clear_active)
		plugins.connect("activate", plugins_active)
		settings.connect("activate", settings_active)
		quit.connect("activate", quit_active)



		menu.add(show_window)
		menu.add(sep1)
		menu.add(open_file)
		menu.add(sep2)
		menu.add(play)
		menu.add(next)
		menu.add(prev)
		menu.add(sep3)
		menu.add(fullscreen)
		menu.add(clear_playlist)
		menu.add(sep4)
		menu.add(plugins)
		menu.add(settings)
		menu.add(sep5)
		menu.add(quit)


		prev.set_sensitive(controls[0].get_property("sensitive"))
		play.set_sensitive(controls[1].get_property("sensitive"))
		next.set_sensitive(controls[2].get_property("sensitive"))


		pop_func = gtk.status_icon_position_menu
		menu.popup(None, None, pop_func, v1, v2, status)
		menu.show_all()

	def set_sub_text(self, text):
		font = subtitles.get_gconf_value('subtitle_font')
		if text == None:
			return

		if text == 'powertotem-disable-text':
			self.subtitle_label.hide()
			return
		else:
			self.subtitle_label.show()

		if self.totem_sub.is_fullscreen() == False:
			font = "Sans 15"

		to_write = "<span color=\"#fff\" font_desc=\"%s\">%s</span>" % (str(font), text)
		self.subtitle_label.set_markup(to_write)

	def preferences_window(self, MenuItem, parent, settings):
		prefsWidget = prefs.prefsDialog(parent, settings, self)

	def shut_menu(self, button):
		self.shut_active = button.get_active()


	def open_subtitles(self, widget):
		Chooser = gtk.FileChooserDialog(_("Select subtitle file"), self.main_window,\
		gtk.FILE_CHOOSER_ACTION_OPEN, ('gtk-cancel', gtk.RESPONSE_CANCEL, 'gtk-open', gtk.RESPONSE_OK), None)

		Result = Chooser.run()

		if Result == gtk.RESPONSE_OK:
			self.OwnSubs = True
			self.OwnSubName = Chooser.get_filename()
		

		Chooser.destroy()
